@@ -6,6 +6,7 @@ Gherkin step implementations for chart features.
 
 from __future__ import absolute_import, print_function
 
+import datetime
 import hashlib
 
 from ast import literal_eval
@@ -20,7 +21,7 @@ from pptx.chart.data import (
 )
 from pptx.dml.color import RGBColor
 from pptx.enum.chart import (
-    XL_AXIS_CROSSES, XL_CHART_TYPE, XL_DATA_LABEL_POSITION,
+    XL_AXIS_CROSSES, XL_CATEGORY_TYPE, XL_CHART_TYPE, XL_DATA_LABEL_POSITION,
     XL_LEGEND_POSITION, XL_MARKER_STYLE
 )
 from pptx.enum.dml import MSO_FILL_TYPE, MSO_THEME_COLOR
@@ -43,12 +44,6 @@ def given_a_axis_type_axis(context, axis_type):
     }[axis_type]
 
 
-@given('a bar plot having known categories')
-def given_a_bar_plot_having_known_categories(context):
-    prs = Presentation(test_pptx('cht-plot-props'))
-    context.plot = prs.slides[2].shapes[0].chart.plots[0]
-
-
 @given('a bar plot {having_or_not} data labels')
 def given_a_bar_plot_having_or_not_data_labels(context, having_or_not):
     slide_idx = {
@@ -149,16 +144,80 @@ def given_a_BubbleChartData_object_with_number_format(context, strval):
     context.chart_data = BubbleChartData(**params)
 
 
+@given('a Categories object containing 3 categories')
+def given_a_Categories_object_containing_3_categories(context):
+    prs = Presentation(test_pptx('cht-category-access'))
+    context.categories = prs.slides[0].shapes[0].chart.plots[0].categories
+
+
+@given('a Categories object having {count} category levels')
+def given_a_Categories_object_having_count_category_levels(context, count):
+    slide_idx = [2, 0, 3, 1][int(count)]
+    slide = Presentation(test_pptx('cht-category-access')).slides[slide_idx]
+    context.categories = slide.shapes[0].chart.plots[0].categories
+
+
+@given('a Categories object having {leafs} categories and {levels} levels')
+def given_a_Categories_obj_having_leafs_and_levels(context, leafs, levels):
+    slide_idx = {
+        (3, 1): 0,
+        (8, 3): 1,
+        (0, 0): 2,
+        (4, 2): 3,
+    }[(int(leafs), int(levels))]
+    slide = Presentation(test_pptx('cht-category-access')).slides[slide_idx]
+    context.categories = slide.shapes[0].chart.plots[0].categories
+
+
+@given('a Categories object with number format {init_nf}')
+def given_a_Categories_object_with_number_format_init_nf(context, init_nf):
+    categories = CategoryChartData().categories
+    if init_nf != 'left as default':
+        categories.number_format = init_nf
+    context.categories = categories
+
+
 @given('a Category object')
 def given_a_Category_object(context):
     context.category = Category(None, None)
 
 
+@given('a Category object having idx value {idx}')
+def given_a_Category_object_having_idx_value_idx(context, idx):
+    cat_offset = int(idx)
+    slide = Presentation(test_pptx('cht-category-access')).slides[0]
+    context.category = slide.shapes[0].chart.plots[0].categories[cat_offset]
+
+
+@given('a Category object having {label}')
+def given_a_Category_object_having_label(context, label):
+    cat_offset = {'label \'Foo\'': 0, 'no label': 1}[label]
+    slide = Presentation(test_pptx('cht-category-access')).slides[0]
+    context.category = slide.shapes[0].chart.plots[0].categories[cat_offset]
+
+
+@given("a category plot")
+def given_a_category_plot(context):
+    prs = Presentation(test_pptx('cht-plot-props'))
+    context.plot = prs.slides[2].shapes[0].chart.plots[0]
+
+
 @given('a CategoryChartData object')
 def given_a_CategoryChartData_object(context):
     context.chart_data = CategoryChartData()
 
 
+@given('a CategoryChartData object having date categories')
+def given_a_CategoryChartData_object_having_date_categories(context):
+    chart_data = CategoryChartData()
+    chart_data.categories = [
+        datetime.date(2016, 12, 27),
+        datetime.date(2016, 12, 28),
+        datetime.date(2016, 12, 29),
+    ]
+    context.chart_data = chart_data
+
+
 @given('a CategoryChartData object with number format {strval}')
 def given_a_CategoryChartData_object_with_number_format(context, strval):
     params = {}
@@ -167,6 +226,20 @@ def given_a_CategoryChartData_object_with_number_format(context, strval):
     context.chart_data = CategoryChartData(**params)
 
 
+@given('a CategoryLevel object containing 4 categories')
+def given_a_CategoryLevel_object_containing_4_categories(context):
+    slide = Presentation(test_pptx('cht-category-access')).slides[1]
+    chart = slide.shapes[0].chart
+    context.category_level = chart.plots[0].categories.levels[1]
+
+
+@given('a chart')
+def given_a_chart(context):
+    prs = Presentation(test_pptx('shp-common-props'))
+    sld = prs.slides[0]
+    context.chart = sld.shapes[6].chart
+
+
 @given('a chart {having_or_not} a legend')
 def given_a_chart_having_or_not_a_legend(context, having_or_not):
     slide_idx = {
@@ -199,36 +272,37 @@ def given_a_chart_of_size_and_type_spec(context, spec):
 @given('a chart of type {chart_type}')
 def given_a_chart_of_type_chart_type(context, chart_type):
     slide_idx, shape_idx = {
-        'Area':                     (0, 0),
-        'Stacked Area':             (0, 1),
-        '100% Stacked Area':        (0, 2),
-        '3-D Area':                 (0, 3),
-        '3-D Stacked Area':         (0, 4),
-        '3-D 100% Stacked Area':    (0, 5),
-        'Clustered Bar':            (1, 0),
-        'Stacked Bar':              (1, 1),
-        '100% Stacked Bar':         (1, 2),
-        'Clustered Column':         (1, 3),
-        'Stacked Column':           (1, 4),
-        '100% Stacked Column':      (1, 5),
-        'Line':                     (2, 0),
-        'Stacked Line':             (2, 1),
-        '100% Stacked Line':        (2, 2),
-        'Marked Line':              (2, 3),
-        'Stacked Marked Line':      (2, 4),
-        '100% Stacked Marked Line': (2, 5),
-        'Pie':                      (3, 0),
-        'Exploded Pie':             (3, 1),
-        'XY (Scatter)':             (4, 0),
-        'XY Lines':                 (4, 1),
-        'XY Lines No Markers':      (4, 2),
-        'XY Smooth Lines':          (4, 3),
-        'XY Smooth No Markers':     (4, 4),
-        'Bubble':                   (5, 0),
-        '3D-Bubble':                (5, 1),
-        'Radar':                    (6, 0),
-        'Marked Radar':             (6, 1),
-        'Filled Radar':             (6, 2),
+        'Area':                        (0, 0),
+        'Stacked Area':                (0, 1),
+        '100% Stacked Area':           (0, 2),
+        '3-D Area':                    (0, 3),
+        '3-D Stacked Area':            (0, 4),
+        '3-D 100% Stacked Area':       (0, 5),
+        'Clustered Bar':               (1, 0),
+        'Stacked Bar':                 (1, 1),
+        '100% Stacked Bar':            (1, 2),
+        'Clustered Column':            (1, 3),
+        'Stacked Column':              (1, 4),
+        '100% Stacked Column':         (1, 5),
+        'Line':                        (2, 0),
+        'Stacked Line':                (2, 1),
+        '100% Stacked Line':           (2, 2),
+        'Marked Line':                 (2, 3),
+        'Stacked Marked Line':         (2, 4),
+        '100% Stacked Marked Line':    (2, 5),
+        'Pie':                         (3, 0),
+        'Exploded Pie':                (3, 1),
+        'XY (Scatter)':                (4, 0),
+        'XY Lines':                    (4, 1),
+        'XY Lines No Markers':         (4, 2),
+        'XY Smooth Lines':             (4, 3),
+        'XY Smooth No Markers':        (4, 4),
+        'Bubble':                      (5, 0),
+        '3D-Bubble':                   (5, 1),
+        'Radar':                       (6, 0),
+        'Marked Radar':                (6, 1),
+        'Filled Radar':                (6, 2),
+        'Line (with date categories)': (7, 0),
     }[chart_type]
     prs = Presentation(test_pptx('cht-chart-type'))
     context.chart = prs.slides[slide_idx].shapes[shape_idx].chart
@@ -359,6 +433,25 @@ def given_a_series(context):
     context.series = prs.slides[0].shapes[0].chart.plots[0].series[0]
 
 
+@given('a series collection for a plot having {count} series')
+def given_a_series_collection_for_a_plot_having_count_series(context, count):
+    prs = Presentation(test_pptx('cht-series-access'))
+    plot = prs.slides[0].shapes[0].chart.plots[0]
+    context.series_collection = plot.series
+    context.series_count = int(count)
+
+
+@given('a series collection for a {type_} chart having {count} series')
+def given_a_series_collection_for_chart_having_series(context, type_, count):
+    slide_idx = {
+        'single-plot': 0,
+        'multi-plot':  1,
+    }[type_]
+    prs = Presentation(test_pptx('cht-series-access'))
+    context.series_collection = prs.slides[slide_idx].shapes[0].chart.series
+    context.series_count = int(count)
+
+
 @given('a series of type {series_type}')
 def given_a_series_of_type_series_type(context, series_type):
     slide_idx = {
@@ -408,6 +501,17 @@ def given_an_axis_having_major_or_minor_unit_of_value(
     context.axis = chart.value_axis
 
 
+@given('an axis of type {cls_name}')
+def given_an_axis_of_type_cls_name(context, cls_name):
+    slide_idx = {
+        'CategoryAxis': 0,
+        'DateAxis':     6,
+    }[cls_name]
+    prs = Presentation(test_pptx('cht-axis-props'))
+    chart = prs.slides[slide_idx].shapes[0].chart
+    context.axis = chart.category_axis
+
+
 @given('an axis not having {major_or_minor} gridlines')
 def given_an_axis_not_having_major_or_minor_gridlines(context, major_or_minor):
     prs = Presentation(test_pptx('cht-axis-props'))
@@ -435,6 +539,17 @@ def given_bar_chart_data_labels_positioned_relation_to_their_data_point(
     context.data_labels = chart.plots[0].data_labels
 
 
+@given('the categories are of type {type_}')
+def given_the_categories_are_of_type(context, type_):
+    label = {
+        'date':  datetime.date(2016, 12, 22),
+        'float': 42.24,
+        'int':   42,
+        'str':   'foobar',
+    }[type_]
+    context.categories.add_category(label)
+
+
 @given('tick labels having an offset of {setting}')
 def given_tick_labels_having_an_offset_of_setting(context, setting):
     slide_idx = {
@@ -786,10 +901,17 @@ def when_I_replace_its_data_with_3_series_of_three_points_each(context):
 
 # then ====================================================
 
-@then("[c.name for c in chart_data.categories] is ['a', 'b', 'c']")
-def then_c_name_for_c_in_chart_data_categories_is_a_b_c(context):
+@then("[c.label for c in chart_data.categories] is ['a', 'b', 'c']")
+def then_c_label_for_c_in_chart_data_categories_is_a_b_c(context):
     chart_data = context.chart_data
-    assert [c.name for c in chart_data.categories] == ['a', 'b', 'c']
+    assert [c.label for c in chart_data.categories] == ['a', 'b', 'c']
+
+
+@then('axis.category_type is XL_CATEGORY_TYPE.{member}')
+def then_axis_category_type_is_XL_CATEGORY_TYPE_member(context, member):
+    expected_value = getattr(XL_CATEGORY_TYPE, member)
+    category_type = context.axis.category_type
+    assert category_type is expected_value, 'got %s' % category_type
 
 
 @then('axis.format is a ChartFormat object')
@@ -848,6 +970,48 @@ def then_bubble_plot_bubble_scale_is_value(context, value):
     )
 
 
+@then('categories[2] is a Category object')
+def then_categories_2_is_a_Category_object(context):
+    type_name = type(context.categories[2]).__name__
+    assert type_name == 'Category', 'got %s' % type_name
+
+
+@then('categories.depth is {value}')
+def then_categories_depth_is_value(context, value):
+    expected_value = int(value)
+    depth = context.categories.depth
+    assert depth == expected_value, 'got %s' % expected_value
+
+
+@then('categories.flattened_labels is a tuple of {leafs} tuples')
+def then_categories_flattened_labels_is_tuple_of_tuples(context, leafs):
+    flattened_labels = context.categories.flattened_labels
+    type_name = type(flattened_labels).__name__
+    length = len(flattened_labels)
+    assert type_name == 'tuple', 'got %s' % type_name
+    assert length == int(leafs), 'got %s' % length
+    for labels in flattened_labels:
+        type_name = type(labels).__name__
+        assert type_name == 'tuple', 'got %s' % type_name
+
+
+@then('categories.levels contains {count} CategoryLevel objects')
+def then_categories_levels_contains_count_CategoryLevel_objs(context, count):
+    expected_idx = int(count) - 1
+    idx = -1
+    for idx, category_level in enumerate(context.categories.levels):
+        type_name = type(category_level).__name__
+        assert type_name == 'CategoryLevel', 'got %s' % type_name
+    assert idx == expected_idx, 'got %s' % idx
+
+
+@then('categories.number_format is {value}')
+def then_categories_number_format_is_value(context, value):
+    expected_value = value
+    number_format = context.categories.number_format
+    assert number_format == expected_value, 'got %s' % number_format
+
+
 @then('category.add_sub_category(name) is a Category object')
 def then_category_add_sub_category_is_a_Category_object(context):
     category = context.category
@@ -855,16 +1019,37 @@ def then_category_add_sub_category_is_a_Category_object(context):
     assert type(sub_category).__name__ == 'Category'
 
 
+@then('category.idx is {value}')
+def then_category_idx_is_value(context, value):
+    expected_value = None if value == 'None' else int(value)
+    idx = context.category.idx
+    assert idx == expected_value, 'got %s' % idx
+
+
+@then('category.label is {value}')
+def then_category_label_is_value(context, value):
+    expected_value = {'\'Foo\'': 'Foo', '\'\'': ''}[value]
+    label = context.category.label
+    assert label == expected_value, 'got %s' % label
+
+
+@then('category_level[2] is a Category object')
+def then_category_level_2_is_a_Category_object(context):
+    type_name = type(context.category_level[2]).__name__
+    assert type_name == 'Category', 'got %s' % type_name
+
+
 @then('category.sub_categories[-1] is the new category')
 def then_category_sub_categories_minus_1_is_the_new_category(context):
     category, sub_category = context.category, context.sub_category
     assert category.sub_categories[-1] is sub_category
 
 
-@then('chart.category_axis is a {type_name} object')
-def then_chart_category_axis_is_an_axis_type_object(context, type_name):
+@then('chart.category_axis is a {cls_name} object')
+def then_chart_category_axis_is_a_cls_name_object(context, cls_name):
     category_axis = context.chart.category_axis
-    assert type(category_axis).__name__ == type_name
+    type_name = type(category_axis).__name__
+    assert type_name == cls_name, 'got %s' % type_name
 
 
 @then('chart.chart_type is {enum_member}')
@@ -890,6 +1075,12 @@ def then_chart_legend_is_a_legend_object(context):
     assert isinstance(chart.legend, Legend)
 
 
+@then('chart.series is a SeriesCollection object')
+def then_chart_series_is_a_SeriesCollection_object(context):
+    type_name = type(context.chart.series).__name__
+    assert type_name == 'SeriesCollection', 'got %s' % type_name
+
+
 @then('chart.value_axis is a ValueAxis object')
 def then_chart_value_axis_is_a_ValueAxis_object(context):
     value_axis = context.chart.value_axis
@@ -991,6 +1182,17 @@ def then_each_series_has_count_values(context, count):
         assert actual_value_count == expected_count
 
 
+@then('each label tuple contains {levels} labels')
+def then_each_label_tuple_contains_levels_labels(context, levels):
+    flattened_labels = context.categories.flattened_labels
+    for labels in flattened_labels:
+        length = len(labels)
+        assert length == int(levels), 'got %s' % levels
+        for label in labels:
+            type_name = type(label).__name__
+            assert type_name == 'str', 'got %s' % type_name
+
+
 @then('gridlines.format is a ChartFormat object')
 def then_gridlines_format_is_a_ChartFormat_object(context):
     gridlines = context.gridlines
@@ -1009,6 +1211,24 @@ def then_gridlines_format_line_is_a_LineFormat_object(context):
     assert type(gridlines.format.line).__name__ == 'LineFormat'
 
 
+@then('iterating categories produces 3 Category objects')
+def then_iterating_categories_produces_3_category_objects(context):
+    categories = context.categories
+    idx = -1
+    for idx, category in enumerate(categories):
+        assert type(category).__name__ == 'Category'
+    assert idx == 2, 'got %s' % idx
+
+
+@then('iterating category_level produces 4 Category objects')
+def then_iterating_category_level_produces_4_Category_objects(context):
+    idx = -1
+    for idx, category in enumerate(context.category_level):
+        type_name = type(category).__name__
+        assert type_name == 'Category', 'got %s' % type_name
+    assert idx == 3, 'got %s' % idx
+
+
 @then('iterating points produces 3 Point objects')
 def then_iterating_points_produces_3_point_objects(context):
     points = context.points
@@ -1018,6 +1238,16 @@ def then_iterating_points_produces_3_point_objects(context):
     assert idx == 2, 'got %s' % idx
 
 
+@then('iterating series_collection produces {count} Series objects')
+def then_iterating_series_collection_produces_count_series(context, count):
+    expected_idx = int(count) - 1
+    idx = -1
+    for idx, series in enumerate(context.series_collection):
+        type_name = type(series).__name__
+        assert type_name.endswith('Series'), 'got %s' % type_name
+    assert idx == expected_idx, 'got %s' % idx
+
+
 @then('legend.font is a Font object')
 def then_legend_font_is_a_Font_object(context):
     legend = context.legend
@@ -1048,6 +1278,19 @@ def then_legend_position_is_value(context, value):
     assert legend.position is expected_position, 'got %s' % legend.position
 
 
+@then('len(categories) is {count}')
+def then_len_categories_is_count(context, count):
+    expected_count = int(count)
+    assert len(context.categories) == expected_count
+
+
+@then('len(category_level) is {count}')
+def then_len_category_level_is_count(context, count):
+    expected_count = int(count)
+    actual_count = len(context.category_level)
+    assert actual_count == expected_count, 'got %s' % actual_count
+
+
 @then('len(chart.series) is {count}')
 def then_len_chart_series_is_count(context, count):
     expected_count = int(count)
@@ -1067,6 +1310,13 @@ def then_len_points_is_3(context):
     assert len(points) == 3
 
 
+@then('len(series_collection) is {count}')
+def then_len_series_collection_is_count(context, count):
+    expected_len = int(count)
+    actual_len = len(context.series_collection)
+    assert actual_len == expected_len, 'got %s' % actual_len
+
+
 @then('len(series.values) is {count} for each series')
 def then_len_series_values_is_count_for_each_series(context, count):
     expected_count = int(count)
@@ -1074,6 +1324,12 @@ def then_len_series_values_is_count_for_each_series(context, count):
         assert len(series.values) == expected_count
 
 
+@then('list(categories) == [\'Foo\', \'\', \'Baz\']')
+def then_list_categories_is_Foo_empty_Baz(context):
+    cats_list = list(context.categories)
+    assert cats_list == ['Foo', '', 'Baz'], 'got %s' % cats_list
+
+
 @then('marker.format is a ChartFormat object')
 def then_marker_format_is_a_ChartFormat_object(context):
     marker = context.marker
@@ -1106,13 +1362,11 @@ def then_marker_style_is_case(context, case):
     assert marker.style == expected_value, 'got %s' % marker.style
 
 
-@then('plot.categories contains the known category strings')
-def then_plot_categories_contains_the_known_category_strings(context):
+@then('plot.categories is a Categories object')
+def then_plot_categories_is_a_Categories_object(context):
     plot = context.plot
-    expected_categories = ('Foo', 'Bar', 'Baz')
-    assert plot.categories == expected_categories, (
-        'got %s' % plot.categories
-    )
+    type_name = type(plot.categories).__name__
+    assert type_name == 'Categories', 'got %s' % type_name
 
 
 @then('plot.gap_width is {value}')
@@ -1184,6 +1438,12 @@ def then_points_2_is_a_Point_object(context):
     assert type(points[2]).__name__ == 'Point'
 
 
+@then('series_collection[2] is a Series object')
+def then_series_collection_2_is_a_Series_object(context):
+    type_name = type(context.series_collection[2]).__name__
+    assert type_name.endswith('Series'), 'got %s' % type_name
+
+
 @then('series.fill.fore_color.rgb is FF6600')
 def then_series_fill_fore_color_rgb_is_FF6600(context):
     fill = context.series.fill
