@@ -4,7 +4,7 @@
 Initialization module for python-pptx
 """
 
-__version__ = '0.6.0'
+__version__ = '0.6.2'
 
 
 import pptx.exc as exceptions
@@ -20,7 +20,10 @@ from pptx.parts.chart import ChartPart
 from pptx.parts.coreprops import CorePropertiesPart
 from pptx.parts.image import ImagePart
 from pptx.parts.presentation import PresentationPart
-from pptx.parts.slide import SlideLayoutPart, SlideMasterPart, SlidePart
+from pptx.parts.slide import (
+    NotesMasterPart, NotesSlidePart, SlideLayoutPart, SlideMasterPart,
+    SlidePart
+)
 
 content_type_to_part_class_map = {
     CT.PML_PRESENTATION_MAIN: PresentationPart,
@@ -28,6 +31,8 @@ content_type_to_part_class_map = {
     CT.PML_TEMPLATE_MAIN:     PresentationPart,
     CT.PML_SLIDESHOW_MAIN:    PresentationPart,
     CT.OPC_CORE_PROPERTIES:   CorePropertiesPart,
+    CT.PML_NOTES_MASTER:      NotesMasterPart,
+    CT.PML_NOTES_SLIDE:       NotesSlidePart,
     CT.PML_SLIDE:             SlidePart,
     CT.PML_SLIDE_LAYOUT:      SlideLayoutPart,
     CT.PML_SLIDE_MASTER:      SlideMasterPart,
