@@ -9,6 +9,8 @@ from __future__ import (
     absolute_import, division, print_function, unicode_literals
 )
 
+import os
+
 from lxml import etree
 
 from .ns import NamespacePrefixedTag
@@ -20,6 +22,20 @@ oxml_parser = etree.XMLParser(remove_blank_text=True, resolve_entities=False)
 oxml_parser.set_element_class_lookup(element_class_lookup)
 
 
+def parse_from_template(template_name):
+    """
+    Return an element loaded from the XML in the template file identified by
+    *template_name*.
+    """
+    thisdir = os.path.split(__file__)[0]
+    filename = os.path.join(
+        thisdir, '..', 'templates', '%s.xml' % template_name
+    )
+    with open(filename, 'rb') as f:
+        xml = f.read()
+    return parse_xml(xml)
+
+
 def parse_xml(xml):
     """
     Return root lxml element obtained by parsing XML character string in
@@ -46,11 +62,12 @@ register_element_cls('a:hlinkHover', CT_Hyperlink)
 
 
 from .chart.axis import (
-    CT_AxisUnit, CT_CatAx, CT_ChartLines, CT_Crosses, CT_LblOffset,
-    CT_Scaling, CT_TickLblPos, CT_TickMark, CT_ValAx
+    CT_AxisUnit, CT_CatAx, CT_ChartLines, CT_Crosses, CT_DateAx,
+    CT_LblOffset, CT_Scaling, CT_TickLblPos, CT_TickMark, CT_ValAx
 )
 register_element_cls('c:catAx',          CT_CatAx)
 register_element_cls('c:crosses',        CT_Crosses)
+register_element_cls('c:dateAx',         CT_DateAx)
 register_element_cls('c:lblOffset',      CT_LblOffset)
 register_element_cls('c:majorGridlines', CT_ChartLines)
 register_element_cls('c:majorTickMark',  CT_TickMark)
@@ -111,10 +128,13 @@ register_element_cls('c:scatterChart',  CT_ScatterChart)
 
 
 from .chart.series import (
-    CT_DPt, CT_NumDataSource, CT_SeriesComposite, CT_StrVal_NumVal_Composite
+    CT_AxDataSource, CT_DPt, CT_Lvl, CT_NumDataSource, CT_SeriesComposite,
+    CT_StrVal_NumVal_Composite
 )
 register_element_cls('c:bubbleSize', CT_NumDataSource)
+register_element_cls('c:cat',        CT_AxDataSource)
 register_element_cls('c:dPt',        CT_DPt)
+register_element_cls('c:lvl',        CT_Lvl)
 register_element_cls('c:pt',         CT_StrVal_NumVal_Composite)
 register_element_cls('c:ser',        CT_SeriesComposite)
 register_element_cls('c:val',        CT_NumDataSource)
@@ -130,6 +150,7 @@ register_element_cls('c:autoUpdate',       CT_Boolean)
 register_element_cls('c:bubble3D',         CT_Boolean)
 register_element_cls('c:crossAx',          CT_UnsignedInt)
 register_element_cls('c:crossesAt',        CT_Double)
+register_element_cls('c:date1904',         CT_Boolean)
 register_element_cls('c:delete',           CT_Boolean)
 register_element_cls('c:idx',              CT_UnsignedInt)
 register_element_cls('c:invertIfNegative', CT_Boolean)
@@ -140,6 +161,7 @@ register_element_cls('c:min',              CT_Double)
 register_element_cls('c:numFmt',           CT_NumFmt)
 register_element_cls('c:order',            CT_UnsignedInt)
 register_element_cls('c:overlay',          CT_Boolean)
+register_element_cls('c:ptCount',          CT_UnsignedInt)
 register_element_cls('c:showLegendKey',    CT_Boolean)
 register_element_cls('c:smooth',           CT_Boolean)
 register_element_cls('c:tx',               CT_Tx)
@@ -181,18 +203,6 @@ register_element_cls('a:solidFill', CT_SolidColorFillProperties)
 register_element_cls('a:srcRect',   CT_RelativeRect)
 
 
-from .slide import (
-    CT_CommonSlideData, CT_Slide, CT_SlideLayout, CT_SlideLayoutIdList,
-    CT_SlideLayoutIdListEntry, CT_SlideMaster
-)
-register_element_cls('p:cSld',           CT_CommonSlideData)
-register_element_cls('p:sld',            CT_Slide)
-register_element_cls('p:sldLayout',      CT_SlideLayout)
-register_element_cls('p:sldLayoutId',    CT_SlideLayoutIdListEntry)
-register_element_cls('p:sldLayoutIdLst', CT_SlideLayoutIdList)
-register_element_cls('p:sldMaster',      CT_SlideMaster)
-
-
 from .presentation import (
     CT_Presentation, CT_SlideId, CT_SlideIdList, CT_SlideMasterIdList,
     CT_SlideMasterIdListEntry, CT_SlideSize
@@ -217,9 +227,15 @@ register_element_cls('p:nvSpPr',   CT_ShapeNonVisual)
 register_element_cls('p:sp',       CT_Shape)
 
 
-from .shapes.connector import CT_Connector, CT_ConnectorNonVisual
-register_element_cls('p:cxnSp',     CT_Connector)
-register_element_cls('p:nvCxnSpPr', CT_ConnectorNonVisual)
+from .shapes.connector import (
+    CT_Connection, CT_Connector, CT_ConnectorNonVisual,
+    CT_NonVisualConnectorProperties
+)
+register_element_cls('a:endCxn',     CT_Connection)
+register_element_cls('a:stCxn',      CT_Connection)
+register_element_cls('p:cNvCxnSpPr', CT_NonVisualConnectorProperties)
+register_element_cls('p:cxnSp',      CT_Connector)
+register_element_cls('p:nvCxnSpPr',  CT_ConnectorNonVisual)
 
 
 from .shapes.graphfrm import (
@@ -277,6 +293,21 @@ register_element_cls('a:tcPr',    CT_TableCellProperties)
 register_element_cls('a:tr',      CT_TableRow)
 
 
+from .slide import (
+    CT_CommonSlideData, CT_NotesMaster, CT_NotesSlide, CT_Slide,
+    CT_SlideLayout, CT_SlideLayoutIdList, CT_SlideLayoutIdListEntry,
+    CT_SlideMaster
+)
+register_element_cls('p:cSld',           CT_CommonSlideData)
+register_element_cls('p:notes',          CT_NotesSlide)
+register_element_cls('p:notesMaster',    CT_NotesMaster)
+register_element_cls('p:sld',            CT_Slide)
+register_element_cls('p:sldLayout',      CT_SlideLayout)
+register_element_cls('p:sldLayoutId',    CT_SlideLayoutIdListEntry)
+register_element_cls('p:sldLayoutIdLst', CT_SlideLayoutIdList)
+register_element_cls('p:sldMaster',      CT_SlideMaster)
+
+
 from .text import (
     CT_RegularTextRun, CT_TextBody, CT_TextBodyProperties,
     CT_TextCharacterProperties, CT_TextField, CT_TextFont, CT_TextLineBreak,
@@ -303,3 +334,7 @@ register_element_cls('a:spcPts',      CT_TextSpacingPoint)
 register_element_cls('a:txBody',      CT_TextBody)
 register_element_cls('c:txPr',        CT_TextBody)
 register_element_cls('p:txBody',      CT_TextBody)
+
+
+from .theme import CT_OfficeStyleSheet
+register_element_cls('a:theme', CT_OfficeStyleSheet)
