@@ -6,15 +6,17 @@ Text-related objects such as TextFrame and Paragraph.
 
 from __future__ import absolute_import, print_function
 
+from ..compat import to_unicode
 from ..dml.fill import FillFormat
 from ..enum.dml import MSO_FILL
+from ..enum.lang import MSO_LANGUAGE_ID
 from ..enum.text import MSO_AUTO_SIZE, MSO_UNDERLINE
 from .fonts import FontFiles
 from .layout import TextFitter
 from ..opc.constants import RELATIONSHIP_TYPE as RT
 from ..oxml.simpletypes import ST_TextWrappingType
 from ..shapes import Subshape
-from ..util import Centipoints, Emu, lazyproperty, Pt, to_unicode
+from ..util import Centipoints, Emu, lazyproperty, Pt
 
 
 class TextFrame(Subshape):
@@ -62,11 +64,11 @@ class TextFrame(Subshape):
     def fit_text(self, font_family='Calibri', max_size=18, bold=False,
                  italic=False, font_file=None):
         """
-        Make the text in this textframe fit entirely within the bounds of its
-        shape by setting word wrap on and applying the "best-fit" font size
-        to all the text it contains. :attr:`TextFrame.auto_size` is set to
-        :attr:`MSO_AUTO_SIZE.NONE`. The font size will not be set larger than
-        *max_size* points. If the path to a matching TrueType font is
+        Make the text in this text frame fit entirely within the bounds of
+        its shape by setting word wrap on and applying the "best-fit" font
+        size to all the text it contains. :attr:`TextFrame.auto_size` is set
+        to :attr:`MSO_AUTO_SIZE.NONE`. The font size will not be set larger
+        than *max_size* points. If the path to a matching TrueType font is
         provided as *font_file*, that font file will be used for the font
         metrics. If *font_file* is |None|, best efforts are made to locate
         a font file with mathching *font_family*, *bold*, and *italic*
@@ -314,6 +316,25 @@ class Font(object):
     def italic(self, value):
         self._rPr.i = value
 
+    @property
+    def language_id(self):
+        """
+        Get or set the language id of this |Font| instance. The language id
+        is a member of the :ref:`MsoLanguageId` enumeration. Assigning |None|
+        removes any language setting, the same behavior as assigning
+        `MSO_LANGUAGE_ID.NONE`.
+        """
+        lang = self._rPr.lang
+        if lang is None:
+            return MSO_LANGUAGE_ID.NONE
+        return self._rPr.lang
+
+    @language_id.setter
+    def language_id(self, value):
+        if value == MSO_LANGUAGE_ID.NONE:
+            value = None
+        self._rPr.lang = value
+
     @property
     def name(self):
         """
