@@ -13,18 +13,22 @@ import pytest
 from pptx.enum.shapes import PP_PLACEHOLDER
 from pptx.parts.presentation import PresentationPart
 from pptx.parts.slide import SlideLayoutPart, SlideMasterPart, SlidePart
-from pptx.shapes.placeholder import LayoutPlaceholder
+from pptx.shapes.base import BaseShape
+from pptx.shapes.placeholder import LayoutPlaceholder, NotesSlidePlaceholder
 from pptx.shapes.shapetree import (
     LayoutPlaceholders, LayoutShapes, MasterPlaceholders, MasterShapes,
-    SlidePlaceholders, SlideShapes
+    NotesSlidePlaceholders, NotesSlideShapes, SlidePlaceholders, SlideShapes
 )
 from pptx.slide import (
-    _BaseSlide, Slide, SlideLayout, SlideLayouts, SlideMaster, SlideMasters,
-    Slides
+    _BaseMaster, _BaseSlide, NotesMaster, NotesSlide, Slide, SlideLayout,
+    SlideLayouts, SlideMaster, SlideMasters, Slides
 )
+from pptx.text.text import TextFrame
 
 from .unitutil.cxml import element, xml
-from .unitutil.mock import call, class_mock, instance_mock, property_mock
+from .unitutil.mock import (
+    call, class_mock, instance_mock, method_mock, property_mock
+)
 
 
 class Describe_BaseSlide(object):
@@ -64,12 +68,234 @@ class Describe_BaseSlide(object):
         return base_slide, new_value, expected_xml
 
 
+class Describe_BaseMaster(object):
+
+    def it_is_a_BaseSlide_subclass(self, subclass_fixture):
+        base_master = subclass_fixture
+        assert isinstance(base_master, _BaseSlide)
+
+    def it_provides_access_to_its_placeholders(self, placeholders_fixture):
+        base_master, MasterPlaceholders_, spTree, placeholders_ = (
+            placeholders_fixture
+        )
+        placeholders = base_master.placeholders
+        MasterPlaceholders_.assert_called_once_with(spTree, base_master)
+        assert placeholders is placeholders_
+
+    def it_provides_access_to_its_shapes(self, shapes_fixture):
+        base_master, MasterShapes_, spTree, shapes_ = shapes_fixture
+        shapes = base_master.shapes
+        MasterShapes_.assert_called_once_with(spTree, base_master)
+        assert shapes is shapes_
+
+    # fixtures -------------------------------------------------------
+
+    @pytest.fixture
+    def placeholders_fixture(self, MasterPlaceholders_, placeholders_):
+        master_element = element('p:sldMaster/p:cSld/p:spTree')
+        base_master = _BaseMaster(master_element, None)
+        spTree = master_element.xpath('//p:spTree')[0]
+        return base_master, MasterPlaceholders_, spTree, placeholders_
+
+    @pytest.fixture
+    def shapes_fixture(self, MasterShapes_, shapes_):
+        master_element = element('p:sldMaster/p:cSld/p:spTree')
+        base_master = _BaseMaster(master_element, None)
+        spTree = master_element.xpath('//p:spTree')[0]
+        return base_master, MasterShapes_, spTree, shapes_
+
+    @pytest.fixture
+    def subclass_fixture(self):
+        return _BaseMaster(None, None)
+
+    # fixture components -----------------------------------
+
+    @pytest.fixture
+    def MasterPlaceholders_(self, request, placeholders_):
+        return class_mock(
+            request, 'pptx.slide.MasterPlaceholders',
+            return_value=placeholders_
+        )
+
+    @pytest.fixture
+    def MasterShapes_(self, request, shapes_):
+        return class_mock(
+            request, 'pptx.slide.MasterShapes', return_value=shapes_
+        )
+
+    @pytest.fixture
+    def placeholders_(self, request):
+        return instance_mock(request, MasterPlaceholders)
+
+    @pytest.fixture
+    def shapes_(self, request):
+        return instance_mock(request, MasterShapes)
+
+
+class DescribeNotesSlide(object):
+
+    def it_can_clone_the_notes_master_placeholders(self, clone_fixture):
+        notes_slide, notes_master_, clone_placeholder_, calls = clone_fixture
+        notes_slide.clone_master_placeholders(notes_master_)
+        assert clone_placeholder_.call_args_list == calls
+
+    def it_provides_access_to_its_shapes(self, shapes_fixture):
+        notes_slide, NotesSlideShapes_, spTree, shapes_ = shapes_fixture
+        shapes = notes_slide.shapes
+        NotesSlideShapes_.assert_called_once_with(spTree, notes_slide)
+        assert shapes is shapes_
+
+    def it_provides_access_to_its_placeholders(self, placeholders_fixture):
+        notes_slide, NotesSlidePlaceholders_, spTree, placeholders_ = (
+            placeholders_fixture
+        )
+        placeholders = notes_slide.placeholders
+        NotesSlidePlaceholders_.assert_called_once_with(spTree, notes_slide)
+        assert placeholders is placeholders_
+
+    def it_provides_access_to_its_notes_placeholder(self, notes_ph_fixture):
+        notes_slide, expected_value = notes_ph_fixture
+        placeholder = notes_slide.notes_placeholder
+        assert placeholder is expected_value
+
+    def it_provides_access_to_its_notes_text_frame(self, notes_tf_fixture):
+        notes_slide, expected_value = notes_tf_fixture
+        text_frame = notes_slide.notes_text_frame
+        assert text_frame is expected_value
+
+    # fixtures -------------------------------------------------------
+
+    @pytest.fixture
+    def clone_fixture(self, notes_master_, clone_placeholder_, shapes_prop_,
+                      shapes_):
+        notes_slide = NotesSlide(None, None)
+        placeholders = notes_master_.placeholders = (
+            BaseShape(element('p:sp/p:nvSpPr/p:nvPr/p:ph{type=body}'), None),
+            BaseShape(element('p:sp/p:nvSpPr/p:nvPr/p:ph{type=dt}'), None),
+        )
+        calls = [call(placeholders[0])]
+        shapes_.clone_placeholder = clone_placeholder_
+        return notes_slide, notes_master_, clone_placeholder_, calls
+
+    @pytest.fixture(params=[
+        (('SLIDE_IMAGE', 'BODY', 'SLIDE_NUMBER'), 1),
+        (('DATE', 'SLIDE_IMAGE', 'FOOTER'), None),
+
+    ])
+    def notes_ph_fixture(self, request, placeholders_prop_):
+        type_names, match = request.param
+        notes_slide = NotesSlide(None, None)
+        placeholders_ = []
+        for type_name in type_names:
+            placeholder_ = instance_mock(
+                request, NotesSlidePlaceholder,
+                name='%s-placeholder' % type_name
+            )
+            placeholder_.placeholder_format.type = getattr(
+                PP_PLACEHOLDER, type_name
+            )
+            placeholders_.append(placeholder_)
+        placeholders_prop_.return_value = placeholders_
+        expected_value = None if match is None else placeholders_[match]
+        return notes_slide, expected_value
+
+    @pytest.fixture(params=[True, False])
+    def notes_tf_fixture(self, request, notes_placeholder_prop_,
+                         placeholder_, text_frame_):
+        has_text_frame = request.param
+        notes_slide = NotesSlide(None, None)
+        if has_text_frame:
+            notes_placeholder_prop_.return_value = placeholder_
+            placeholder_.text_frame = text_frame_
+            expected_value = text_frame_
+        else:
+            notes_placeholder_prop_.return_value = None
+            expected_value = None
+        return notes_slide, expected_value
+
+    @pytest.fixture
+    def placeholders_fixture(self, NotesSlidePlaceholders_, placeholders_):
+        notes = element('p:notes/p:cSld/p:spTree')
+        notes_slide = NotesSlide(notes, None)
+        spTree = notes.xpath('//p:spTree')[0]
+        return notes_slide, NotesSlidePlaceholders_, spTree, placeholders_
+
+    @pytest.fixture
+    def shapes_fixture(self, NotesSlideShapes_, shapes_):
+        notes = element('p:notes/p:cSld/p:spTree')
+        notes_slide = NotesSlide(notes, None)
+        spTree = notes.xpath('//p:spTree')[0]
+        return notes_slide, NotesSlideShapes_, spTree, shapes_
+
+    # fixture components ---------------------------------------------
+
+    @pytest.fixture
+    def clone_placeholder_(self, request):
+        return method_mock(request, NotesSlideShapes, 'clone_placeholder')
+
+    @pytest.fixture
+    def notes_master_(self, request):
+        return instance_mock(request, NotesMaster)
+
+    @pytest.fixture
+    def notes_placeholder_prop_(self, request, placeholder_):
+        return property_mock(
+            request, NotesSlide, 'notes_placeholder',
+            return_value=placeholder_
+        )
+
+    @pytest.fixture
+    def NotesSlidePlaceholders_(self, request, placeholders_):
+        return class_mock(
+            request, 'pptx.slide.NotesSlidePlaceholders',
+            return_value=placeholders_
+        )
+
+    @pytest.fixture
+    def NotesSlideShapes_(self, request, shapes_):
+        return class_mock(
+            request, 'pptx.slide.NotesSlideShapes', return_value=shapes_
+        )
+
+    @pytest.fixture
+    def placeholder_(self, request):
+        return instance_mock(request, NotesSlidePlaceholder)
+
+    @pytest.fixture
+    def placeholders_(self, request):
+        return instance_mock(request, NotesSlidePlaceholders)
+
+    @pytest.fixture
+    def placeholders_prop_(self, request, placeholders_):
+        return property_mock(
+            request, NotesSlide, 'placeholders', return_value=placeholders_
+        )
+
+    @pytest.fixture
+    def shapes_(self, request):
+        return instance_mock(request, NotesSlideShapes)
+
+    @pytest.fixture
+    def shapes_prop_(self, request, shapes_):
+        return property_mock(
+            request, NotesSlide, 'shapes', return_value=shapes_
+        )
+
+    @pytest.fixture
+    def text_frame_(self, request):
+        return instance_mock(request, TextFrame)
+
+
 class DescribeSlide(object):
 
     def it_is_a_BaseSlide_subclass(self, subclass_fixture):
         slide = subclass_fixture
         assert isinstance(slide, _BaseSlide)
 
+    def it_knows_whether_it_has_a_notes_slide(self, has_notes_slide_fixture):
+        slide, expected_value = has_notes_slide_fixture
+        assert slide.has_notes_slide is expected_value
+
     def it_knows_its_slide_id(self, slide_id_fixture):
         slide, expected_value = slide_id_fixture
         assert slide.slide_id == expected_value
@@ -92,8 +318,18 @@ class DescribeSlide(object):
         slide, slide_layout_ = layout_fixture
         assert slide.slide_layout is slide_layout_
 
+    def it_provides_access_to_its_notes_slide(self, notes_slide_fixture):
+        slide, notes_slide_ = notes_slide_fixture
+        assert slide.notes_slide is notes_slide_
+
     # fixtures -------------------------------------------------------
 
+    @pytest.fixture
+    def has_notes_slide_fixture(self, part_prop_, slide_part_):
+        slide = Slide(None, None)
+        expected_value = slide_part_.has_notes_slide = 42
+        return slide, expected_value
+
     @pytest.fixture
     def layout_fixture(self, slide_layout_, part_prop_, slide_part_):
         slide = Slide(None, None)
@@ -101,6 +337,12 @@ class DescribeSlide(object):
         slide_part_.slide_layout = slide_layout_
         return slide, slide_layout_
 
+    @pytest.fixture
+    def notes_slide_fixture(self, notes_slide_, part_prop_, slide_part_):
+        slide = Slide(None, None)
+        slide_part_.notes_slide = notes_slide_
+        return slide, notes_slide_
+
     @pytest.fixture
     def placeholders_fixture(self, SlidePlaceholders_, placeholders_):
         sld = element('p:sld/p:cSld/p:spTree')
@@ -126,7 +368,11 @@ class DescribeSlide(object):
     def subclass_fixture(self):
         return Slide(None, None)
 
-    # fixture components -----------------------------------
+    # fixture components ---------------------------------------------
+
+    @pytest.fixture
+    def notes_slide_(self, request):
+        return instance_mock(request, NotesSlide)
 
     @pytest.fixture
     def part_prop_(self, request, slide_part_):
@@ -352,7 +598,8 @@ class DescribeSlideLayout(object):
             self, request, placeholders_prop_, placeholder_, placeholder_2_):
         ph_types, expected_indices = request.param
         slide_layout = SlideLayout(None, None)
-        placeholder_.ph_type, placeholder_2_.ph_type = ph_types
+        placeholder_.element.ph_type = ph_types[0]
+        placeholder_2_.element.ph_type = ph_types[1]
         _placeholders = (placeholder_, placeholder_2_)
         expected_placeholders = [
             _placeholders[idx] for idx in expected_indices
@@ -520,23 +767,9 @@ class DescribeSlideLayouts(object):
 
 class DescribeSlideMaster(object):
 
-    def it_is_a_BaseSlide_subclass(self, subclass_fixture):
-        slide = subclass_fixture
-        assert isinstance(slide, _BaseSlide)
-
-    def it_provides_access_to_its_placeholders(self, placeholders_fixture):
-        slide_master, MasterPlaceholders_, spTree, placeholders_ = (
-            placeholders_fixture
-        )
-        placeholders = slide_master.placeholders
-        MasterPlaceholders_.assert_called_once_with(spTree, slide_master)
-        assert placeholders is placeholders_
-
-    def it_provides_access_to_its_shapes(self, shapes_fixture):
-        slide_master, MasterShapes_, spTree, shapes_ = shapes_fixture
-        shapes = slide_master.shapes
-        MasterShapes_.assert_called_once_with(spTree, slide_master)
-        assert shapes is shapes_
+    def it_is_a_BaseMaster_subclass(self, subclass_fixture):
+        slide_master = subclass_fixture
+        assert isinstance(slide_master, _BaseMaster)
 
     def it_provides_access_to_its_slide_layouts(self, layouts_fixture):
         slide_master, SlideLayouts_, sldLayoutIdLst, slide_layouts_ = (
@@ -555,47 +788,12 @@ class DescribeSlideMaster(object):
         sldMasterIdLst = sldMaster.sldLayoutIdLst
         return slide_master, SlideLayouts_, sldMasterIdLst, slide_layouts_
 
-    @pytest.fixture
-    def placeholders_fixture(self, MasterPlaceholders_, placeholders_):
-        sldMaster = element('p:sldMaster/p:cSld/p:spTree')
-        slide_master = SlideMaster(sldMaster, None)
-        spTree = sldMaster.xpath('//p:spTree')[0]
-        return slide_master, MasterPlaceholders_, spTree, placeholders_
-
-    @pytest.fixture
-    def shapes_fixture(self, MasterShapes_, shapes_):
-        sldMaster = element('p:sldMaster/p:cSld/p:spTree')
-        slide_master = SlideMaster(sldMaster, None)
-        spTree = sldMaster.xpath('//p:spTree')[0]
-        return slide_master, MasterShapes_, spTree, shapes_
-
     @pytest.fixture
     def subclass_fixture(self):
         return SlideMaster(None, None)
 
     # fixture components -----------------------------------
 
-    @pytest.fixture
-    def MasterPlaceholders_(self, request, placeholders_):
-        return class_mock(
-            request, 'pptx.slide.MasterPlaceholders',
-            return_value=placeholders_
-        )
-
-    @pytest.fixture
-    def MasterShapes_(self, request, shapes_):
-        return class_mock(
-            request, 'pptx.slide.MasterShapes', return_value=shapes_
-        )
-
-    @pytest.fixture
-    def placeholders_(self, request):
-        return instance_mock(request, MasterPlaceholders)
-
-    @pytest.fixture
-    def shapes_(self, request):
-        return instance_mock(request, MasterShapes)
-
     @pytest.fixture
     def SlideLayouts_(self, request, slide_layouts_):
         return class_mock(
